/*
 * Name:    fit_to_max.c
 *
 * Purpose: Find the position at which a scaler value is maximum.
 *
 *--------------------------------------------------------------------
 *
 * Copyright 1999 Illinois Institute of Technology
 *
 * See the file "LICENSE" for information on usage and redistribution
 * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 *
 */


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>

int
main( int argc, char *argv[] )
{
	FILE *datafile;
	char buffer[80];
	double *xarray, *yarray;
	double max_position, max_value;
	unsigned long i, num_measurements;
	int num_items;

	if ( argc != 2 ) {
		fprintf( stderr, "Usage: fit_to_max filename\n" );

		printf("0 0\n");
		exit(0);
	}

	/* Open the datafile. */

	datafile = fopen( argv[1], "r" );

	if ( datafile == NULL ) {
		fprintf( stderr, "Can't open scan datafile '%s'\n", argv[1] );

		printf("0 0\n");
		exit(0);
	}

	/* How many measurements are there in the file? */

	num_measurements = 0;

	while ( 1 ) {
		fgets( buffer, sizeof buffer, datafile );

		if ( feof(datafile) || ferror(datafile) )
			break;

		num_measurements++;
	}

	rewind(datafile);

	/* Create arrays to store the scan data in. */

	xarray = (double *) malloc( num_measurements * sizeof(double) );

	if ( xarray == NULL ) {
		fprintf( stderr,
			"Attempt to allocate %lu element X array failed.\n",
			num_measurements );

		printf("0 0\n");
		exit(0);
	}

	yarray = (double *) malloc( num_measurements * sizeof(double) );

	if ( yarray == NULL ) {
		fprintf( stderr,
			"Attempt to allocate %lu element Y array failed.\n",
			num_measurements );

		printf("0 0\n");
		exit(0);
	}

	/* Read the data in. */

	for ( i = 0; i < num_measurements; i++ ) {
		fgets( buffer, sizeof buffer, datafile );

		if ( feof(datafile) || ferror(datafile) ) {
			fprintf( stderr,
	"Error while attempting to read line %lu of the scan datafile '%s'\n",
				i, argv[1] );

			printf("0 0\n");
			exit(0);
		}

		num_items = sscanf( buffer, "%lg %lg",
				&xarray[i], &yarray[i] );

		if ( num_items != 2 ) {
			fprintf( stderr,
	"Error parsing line %lu of the scan datafile '%s'.  buffer = '%s'\n",
				i, argv[1], buffer );

			printf("0 0\n");
			exit(0);
		}
	}

	fclose( datafile );

	/* Now fit the data in the datafile. */

	max_position = -DBL_MAX;
	max_value = -DBL_MAX;

	for ( i = 0; i < num_measurements; i++ ) {

		if ( yarray[i] > max_value ) {
			max_position = xarray[i];
			max_value = yarray[i];
		}
	}

	printf("1 %g %g\n", max_position, max_value);
	exit(0);
}

